$TeamDeployableMax[MissileSiloPack]   = 9999;

datablock ShapeBaseImageData(MissileSiloDeployableImage) {
    mass = 1;
	emap = true;
	shapeFile = "stackable1s.dts";
	item = MissileSiloPack;
	mountPoint = 1;
	offset = "0 0 0";
	deployed = MissileSilo;
	heatSignature = 0;

	stateName[0] = "Idle";
	stateTransitionOnTriggerDown[0] = "Activate";

	stateName[1] = "Activate";
	stateScript[1] = "onActivate";
	stateTransitionOnTriggerUp[1] = "Idle";

	isLarge = true;
	maxDepSlope = 360;
	deploySound = StationDeploySound;

	minDeployDis = 0.5;
	maxDeployDis = 5.0;

    cost = 2500;
};

datablock ItemData(MissileSiloPack) {
	className = Pack;
	catagory = "Deployables";
	shapeFile = "stackable1s.dts";
    mass = 1;
	elasticity = 0.2;
	friction = 0.6;
	pickupRadius = 1;
	rotate = false;
	image = "MissileSiloDeployableImage";
	pickUpName = "a missile silo pack";
	heatSignature = 0;

	computeCRC = true;
	emap = true;
};

datablock StaticShapeData(MissileSilo) {
	className = "crate";
	shapeFile = "nexuscap.dts";

	maxDamage      = 5.0;
	destroyedLevel = 5.0;
	disabledLevel  = 4.7;

	explosion      = HandGrenadeExplosion;
	expDmgRadius = 1.0;
	expDamage = 0.05;
	expImpulse = 200;

	dynamicType = $TypeMasks::StaticShapeObjectType;
	deployedObject = true;
	cmdCategory = "DSupport";
	cmdIcon = CMDSensorIcon;
	cmdMiniIconName = "commander/MiniIcons/com_deploymotionsensor";
	targetNameTag = 'Missile';
    targetTypeTag = 'Silo';
	deployAmbientThread = true;
	debrisShapeName = "debris_generic_small.dts";
	debris = DeployableDebris;
	heatSignature = 0;
    needsPower = false;
};

function MissileSiloPack::onPickup(%this,%obj,%shape,%amount) {
	// created to prevent console errors
}

function MissileSiloDeployableImage::onDeploy(%item,%plyr,%slot) {
	%className = "StaticShape";

	%item.surfacePt = vectorAdd(%item.surfacePt,vectorScale(%item.surfaceNrm,0.4));

	%playerVector = vectorNormalize(getWord(%plyr.getEyeVector(),1) SPC -1 * getWord(%plyr.getEyeVector(),0) SPC "0");
	%item.surfaceNrm2 = %playerVector;

	if (vAbs(floorVec(%item.surfaceNrm,100)) $= "0 0 1")
		%item.surfaceNrm2 = vectorScale(%playerVector,-1);
	else
		%item.surfaceNrm2 = vectorNormalize(vectorCross(%item.surfaceNrm,"0 0 1"));

	%rot = fullRot(vectorScale(%item.surfaceNrm,-1),%item.surfaceNrm2);

	%deplObj = new (%className)() {
		dataBlock = MissileSilo;
		scale = "1 1 1";
		deployed = true;
	};

    //create Control Pad
    %deplObj.PAD = new StaticShape() {
        datablock = "ControlStation";
        position = vectorAdd(%item.surfacePt, "4 4 0");
        team = %plyr.client.team;
    };
    %deplObj.PAD.team = %plyr.client.team;
    %deplObj.PAD.hudToPush = "MissileSilo";
    %deplObj.PAD.ControllingObj = %deplObj;

    //team stuff
   if (%deplObj.PAD.getTarget() != -1)
      setTargetSensorGroup(%deplObj.PAD.getTarget(), %plyr.client.team);

	// set orientation
	%deplObj.setTransform(%item.surfacePt SPC %rot);

	// set team, owner, and handle
	%deplObj.team = %plyr.client.team;
	%deplObj.setOwner(%plyr);
	%deplObj.PAD.setOwner(%plyr);

	// set power frequency
	%deplObj.powerFreq = %plyr.powerFreq;

	// set the sensor group if it needs one
	if (%deplObj.getTarget() != -1)
		setTargetSensorGroup(%deplObj.getTarget(),%plyr.client.team);

	setTargetName(%deplObj.target,addTaggedString("Missile Silo"));

	// place the deployable in the MissionCleanup/Deployables group (AI reasons)
	addToDeployGroup(%deplObj);
    addToDeployGroup(%deplObj.PAD);

	//let the AI know as well...
	AIDeployObject(%plyr.client,%deplObj);

	// play the deploy sound
	serverPlay3D(%item.deploySound,%deplObj.getTransform());

	// increment the team count for this deployed object
	$TeamDeployedCount[%plyr.team,%item.item]++;

	addDSurface(%item.surface,%deplObj);

    %plyr.client.credits -= %item.cost;

	// Power object
	checkPowerObject(%deplObj);
 
   	%center = %deplObj.getEdge("0 0 0");
	%deplObj.setTransform(remoteRotate(%deplObj,"1 0 0 3.141593", %deplObj,"0 0 0"));
	%deplObj.setEdge(%center, "0 0 0");

    //woohoo!
    for(%i = 0; %i < ClientGroup.getCount(); %i++) {
       %cl = ClientGroup.getObject(%i);
       if(%cl != %plyr.client) {
          messageClient(%cl, 'MsgClient', "\c5DEFCON ALERT: A Missile Silo Has Been Detected! ~wfx/misc/red_alert.wav");
       }
    }

	return %deplObj;
}

function MissileSiloDeployableImage::onMount(%data,%obj,%node) {
	%obj.hasSilo = true; // set for telecheck
}

function MissileSiloDeployableImage::onUnmount(%data,%obj,%node) {
	%obj.hasSilo = "";
}

//MODIFIED SILO LAUNCH FUNCTION
function GameBase::LaunchSiloMissile(%obj) {
   %data = $mpm_load[%obj.load];
   %barrelt = %obj.getPosition();
   %up = Realvec(%obj,"0 0 1");

   %start = VectorAdd(%barrelt,VectorScale(%up,5));
   %end = %obj.targetLoc;
   
   %forward = VectorNormalize(VectorCross(VectorCross(%up,VectorSub(%end,%start)),%up));

   if (%data.offset !$= "")
      %end = VectorAdd(%end,VectorAdd(VectorScale(%forward,GetWord(%data.offset,0)),VectorScale(%up,GetWord(%data.offset,1))));

   %missileblock = %data.missile;

   %solution = mpm_calc(%start,%end,%up,%missileblock);
   if (%solution != -1) {
      %ltime = GetWord(%solution,0)*1000;
      %time = GetWord(%solution,1)*1000;
      %loc = GetWord(%solution,2);
      %vector = GetWords(%solution,3,5);
      %a = GetWord(%solution,6);
      %p = Launch_Mpm(%start,%up,"0 0 0",%ltime+200,%missileblock);
      %p.team = %obj.team;
      %p.source = %obj;
      %p.traject = %start SPC %up SPC %loc SPC %a SPC %vector SPC %time;
      %p.load = %data;
      %p.targetlocation = %end;
      %p.owner = %obj;
      %data.schedule(%ltime,"Stage2",%p);
      %data.Stage1(%p);

      %obj.missile = "";
   }

}
