//Collectors
datablock AudioProfile(CashInSound) {
   filename    = "fx/misc/bounty_bonus.wav";
   description = AudioClosest3d;
   preload = true;
};

datablock PlayerData(LightCollectorArmor) : LightMaleBiodermArmor {
   maxDamage = 1.0;
   minImpactSpeed = 50;
   speedDamageScale = 0.015;

   MaxCollect = 75;
   CollectTicks = 5;

   speed = 2300;
};

//Commander Anderson
datablock PlayerData(MediumCollectorArmor) : MediumMaleBiodermArmor {
   maxDamage = 1.5;
   minImpactSpeed = 50;
   speedDamageScale = 0.015;

   MaxCollect = 100;
   CollectTicks = 10;

   speed = 2750;
};


datablock PlayerData(HeavyCollectorArmor) : HeavyMaleBiodermArmor {
   maxDamage = 2.0;
   minImpactSpeed = 50;
   speedDamageScale = 0.015;

   MaxCollect = 120;
   CollectTicks = 10;

   speed = 3300;
};

function SpawnCollector(%client, %type) {
   if(%type == 1) {
   	  %collector = new player(){
	     Datablock = "LightCollectorArmor";
   	  };
      %name = "Light Resource Gatherer";
   }
   else if(%type == 2) {
   	  %collector = new player(){
	     Datablock = "HeavyCollectorArmor";
   	  };
      %name = "Heavy Resource Gatherer";
   }
   else if(%type == 3) {
   	  %collector = new player(){
	     Datablock = "MediumCollectorArmor";
   	  };
      %name = "Medium Resource Gatherer";
   }
   
   %collector.name = %name;
   
   %collector.load = 0;

   %collector.target = createTarget(%collector, %name, "", "Derm3", '', %collector.team, PlayerSensor);
   setTargetName(%collector.target, addtaggedstring(%name));
   setTargetSkin(%collector.target, 'Horde');

   %collector.ReturningToBase = 0;
   %collector.Collecting = 0;
   %collector.BearingSet = 0;

   %client.OnDeployUnit(%collector);

   schedule(500,0,"CollectorFindResources",%collector);
}

//Collector AI
function CollectorFindResources(%collector) {
   if(!isAlive(%collector)) {     //we still alive?
   //   echo("dead");
      return;
   }
   if(%collector.ReturningToBase) {  //are we currently returning a load?
     // echo("returning");
      schedule(3000, 0, "CollectorFindResources", %collector);
      return;
   }
   if(%collector.Collecting) { //we are already collecting the goods
    //  echo("collecting");
      schedule(3000, 0, "CollectorFindResources", %collector);
      return;
   }
   if(%collector.SideSwipe) { //we are already moving to a site
   //   echo("bearing");
      schedule(3000, 0, "CollectorFindResources", %collector);
      return;
   }
   if(%collector.BearingSet) { //we are already moving to a site
   //   echo("bearing");
      schedule(3000, 0, "CollectorFindResources", %collector);
      return;
   }
   //Ok, now we seek out the goods
   %group = nameToID("DefconResources");
   if(%group == -1) {  //blocks really bad things
      error("EXTREME ERROR: No RESOURCES, Creating Group NOW!!!.");
      %group = new SimGroup("DefconResources");
      schedule(1000, 0, "placeRandomResourceZones");
      schedule(3000, 0, "CollectorFindResources", %collector);
      return;
   }
   else {
      %wbpos = %collector.getworldboxcenter();
      %closestDistance = 32767;
      for(%i = 0; %i < %group.getCount(); %i++) {
	     %mine = %group.getObject(%i);
	     if(isObject(%mine)){
	        %testPos = %mine.getWorldBoxCenter();
	        %distance = vectorDist(%wbpos, %testPos);
	        if (%distance > 0 && %distance < %closestDistance && %mine.isResourceMine) {
	   	       %position = %mine.getPosition();
               %closestDistance = %distance;
	        }
         }
      }
      if(%position) {
       //  echo("moving");
         CollectorMove(%collector, %position); // lets go!
      }
      else {
        // echo("no mines found");
      }
   }
   schedule(3000, 0, "CollectorFindResources", %collector);
}

function GetDirectionToMove(%collector, %pos){
//   echo(%pos);
   %cPos = %collector.getPosition();
   %vector = vectorNormalize(vectorSub(%pos, %cpos));
   %v1 = getword(%vector, 0);
   %v2 = getword(%vector, 1);
   %v3 = getword(%vector, 2);
   %nv1 = (%v2);
   %nv2 = (%v1*-1);
   %set = %nv1@" "@%nv2@" "@%v3;
   %collector.setRotation(fullrot("0 0 0", %Set));

   return %vector;
}

function CollectorCollectLoop(%obj) {
   if(!isAlive(%obj)) {
	  return;
   }
   serverPlay3D(SingeSound, %obj.getPosition());
   %obj.load += %obj.getDatablock().CollectTicks;
   %max = %obj.getDatablock().MaxCollect;
   if(%obj.load >= %max) { //alright.. done collecting, lets head home
      %obj.load = %max;
      %obj.Collecting = 0;
      %obj.SideSwipe = 1;
      CollectorMove(%obj, VectorAdd(%obj.getPosition(), ""@getRandom(-50,50)@" 40 0"));
      return;
   }
   schedule(1500, 0, "CollectorCollectLoop", %obj);
}

function CollectorMove(%collector, %targetLocation) {
   if(!isAlive(%collector)) {
	  return;
   }
   %pos = %collector.getworldboxcenter();
   //echo(""@%pos@" - "@%targetLocation@" - "@vectorDist(%pos,%targetLocation)@"");
   %collector.BearingSet = 1;
   if(vectorDist(%pos, %targetLocation) < 11) {
      //returned to base... time to drop off the g00ds
      if(%collector.ReturningToBase) {
         if(%collector.owner.getCommander() $= "Anderson") {
            %togive = TapResources(%collector.load);
            %collector.owner.credits += %togive;
            messageClient(%collector.owner, 'MsgDropOff', "\c5Collector: Load of "@%collector.load@", tapped to "@%togive@" dropped off, you now have "@%collector.owner.credits@" credits.");
            %collector.load = 0;
         }
         else {
            %collector.owner.credits += %collector.load;
            messageClient(%collector.owner, 'MsgDropOff', "\c5Collector: Load of "@%collector.load@" dropped off, you now have "@%collector.owner.credits@" credits.");
            %collector.load = 0;
         }
         // set status
         %collector.ReturningToBase = 0;
         %collector.BearingSet = 0; // the loops will kick in, right about....... now!
         ServerPlay3D(CashInSound, %collector.getPosition());
         return;
      }
      //must be a mine
      else if(%collector.sideSwipe) {
         %collector.SideSwipe = 0;
         %collector.ReturningToBase = 1;
         if(!isObject(%collector.home)) {
            %collector.home = %collector.owner.CP;
         }
         CollectorMove(%collector, %collector.home.getPosition());
         return;
      }
      else {
         %collector.Collecting = 1;
         CollectorCollectLoop(%collector);
         return;
      }
   }

   %collector.setActionThread("scoutRoot",true); //since only light armor does this, we don't worry about any checks for the other armors
   %fmultiplier = %collector.getDatablock().Speed;

   %vector = GetDirectionToMove(%collector, %targetLocation);

   %vector = vectorscale(%vector, %Fmultiplier);
   %x = Getword(%vector,0);
   %y = Getword(%vector,1);

   %vector = %x@" "@%y@" 0";
   %collector.applyImpulse(%pos, %vector);

   //if(%collector.getForwardVector() $= "0 0 0") { //old method
   %muzzlePos = %collector.getMuzzlePoint(0);
   %muzzleVec = %collector.getMuzzleVector(0);
   %endPos    = VectorAdd(%muzzlePos, VectorScale(%muzzleVec, 4));
   %Mask = $TypeMasks::PlayerObjectType | $TypeMasks::StaticShapeObjectType;

   //stuck?
   %hit = ContainerRayCast(%muzzlePos, %endPos, %Mask, %collector);
   if(%hit) {
      %collector.setVelocity(""@getRandom(-5,5)@" "@getRandom(-5,5)@" "@getRandom(1,5)@"");
      //now we arent :)
   }
   //}

   %collector.moveloop = schedule(500, %collector, "CollectorMove", %collector, %targetLocation);
}

//ORDERS   (none, deselect)
function LightCollectorArmor::OnSelectUnit(%data, %scout, %obj) {
   %obj.SelectedUnit = "";
   BottomPrint(%obj.client, "Unit has pre-set Orders, De-Selected.", 1, 1);
}

function MediumCollectorArmor::OnSelectUnit(%data, %scout, %obj) {
   %obj.SelectedUnit = "";
   BottomPrint(%obj.client, "Unit has pre-set Orders, De-Selected.", 1, 1);
}

function HeavyCollectorArmor::OnSelectUnit(%data, %scout, %obj) {
   %obj.SelectedUnit = "";
   BottomPrint(%obj.client, "Unit has pre-set Orders, De-Selected.", 1, 1);
}
