//UNIT: DISC SOLDIER
//COST: 750, DEFCON 4
//DESCRIPTION: Disc Soldiers attack enemy units / bases

datablock PlayerData(DiscSoldierArmor) : MediumMaleHumanArmor {
   maxDamage = 1.5;
   minImpactSpeed = 50;
   speedDamageScale = 0.015;

   speed = 2400;
};

function SpawnDiscSoldier(%client) {
   %DiscSoldier = new player(){
      Datablock = "DiscSoldierArmor";
   };
   %name = "Disc Soldier";

   %DiscSoldier.name = %name;

   %DiscSoldier.setInventory(Disc, 1, true);
   %DiscSoldier.setInventory(DiscAmmo, 99999, true);
   %DiscSoldier.use(Disc);

   %DiscSoldier.target = createTarget(%DiscSoldier, %name, "", "Male2", '', %DiscSoldier.team, PlayerSensor);
   setTargetName(%DiscSoldier.target, addtaggedstring(%name));
   setTargetSkin(%DiscSoldier.target, 'BEagle');

   %DiscSoldier.BearingSet = 0;

   %client.OnDeployUnit(%DiscSoldier);
}

function DiscSoldierArmor::OnSelectUnit(%data, %DiscSoldier, %obj) {
   %DiscSoldier.nameToPush = "Disc Soldier";
   %obj.order[0] = "Move [PTM]";
   %obj.order[1] = "Attack Enemy Object [PTSL]";
   %obj.order[2] = "Patrol Area [PTO]";
   %obj.order[3] = "Cancel Orders [PTO]";
}

function DiscSoldierArmor::DisplayOrders(%data, %unit, %obj) {
   BottomPrint(%obj.client, "Unit: "@%unit.nameToPush@"("@%unit@") \n ORDER: "@%obj.order[%obj.TLMineMode]@"", 2, 2);
}

function DiscSoldierArmor::ProcessOrders(%data, %DiscSoldier, %orders, %hitObj, %Position) {
   if(!isAlive(%DiscSoldier)) {
      return;
   }
   //
   if((%DiscSoldier.orders || %DiscSoldier.Patrolling) && %orders != 3) {  //we cannot do two things at once
      bottomPrint(%DiscSoldier.owner, "Cannot perform a second order, must cancel first", 1, 1);
      return;
   }
   else {
      switch(%orders) {
         case 0:
            %DiscSoldier.orders = 1;
            DiscSoldierMove(%DiscSoldier, %Position, "");
         case 1:
            if(%hitObj.team == %DiscSoldier.team || %hitObj.team == 0) {
               messageClient(%DiscSoldier.owner, 'msgInvalid', "\c5Disc Soldier: That's not a legit target... sir!");
               %DiscSoldier.orders = 0;
               return;
            }
            else {
               %DiscSoldier.orders = 1;
               DiscSoldierMove(%DiscSoldier, %Position, %hitObj);
            }
         case 2:
            %DiscSoldier.Patrolling = 1;
            %DiscSoldier.PatrolLoop = DiscSoldierPatrolLoop(%DiscSoldier, %DiscSoldier.getPosition());
         case 3:
            %DiscSoldier.orders = 0;
            %DiscSoldier.attacking = 0;
            %DiscSoldier.Moving = 0;
            %DiscSoldier.Patrolling = 0;
            cancel(%DiscSoldier.PatrolLoop);
         default:
            echo("Invalid orders");
            return;
      }
   }
}

function DiscSoldierMove(%DiscSoldier, %targetLocation, %target) {
   if(!isAlive(%DiscSoldier)) {
	  return;
   }
   if(!%DiscSoldier.orders) {
      return;
   }
   if(%DiscSoldier.attacking) {
      return;
   }
   %pos = %DiscSoldier.getworldboxcenter();
   //echo(""@%pos@" - "@%targetLocation@" - "@vectorDist(%pos,%targetLocation)@"");
   %fmultiplier = %DiscSoldier.getDatablock().Speed;

   if(!%target || %target $= "") {
      %vector = GetDirectionToMove(%DiscSoldier, %targetLocation);

      %vector = vectorscale(%vector, %Fmultiplier);
      %x = Getword(%vector,0);
      %y = Getword(%vector,1);

      %vector = %x@" "@%y@" 0";
      %DiscSoldier.applyImpulse(%pos, %vector);

      //reached target, lets move on
      if(vectorDist(%pos, %targetLocation) < 10) {
         %DiscSoldier.orders = 0;
         %DiscSoldier.Moving = 0;
         return;
      }

      //if(%DiscSoldier.getForwardVector() $= "0 0 0") { //old method
      %muzzlePos = %DiscSoldier.getEyePoint();
      %muzzleVec = %DiscSoldier.getEyeVector();
      %endPos    = VectorAdd(%muzzlePos, VectorScale(%muzzleVec, 2));
      %Mask = $TypeMasks::PlayerObjectType;

      //stuck?
      %hit = ContainerRayCast(%muzzlePos, %endPos, %Mask, %DiscSoldier);
      if(%hit) {
         %DiscSoldier.setVelocity(""@getRandom(-5,5)@" "@getRandom(-5,5)@" "@getRandom(1,5)@"");
         //now we arent :)
      }
   }
   else {
      if(!isAlive(%target)) {
         %DiscSoldier.orders = 0;
         %DiscSoldier.attacking = 0;
         %DiscSoldier.Moving = 0;
         %DiscSoldier.setImageTrigger(0, false);
         return;
      }
      %targetLocation = %target.getPosition();
      %vector = GetDirectionToMove(%DiscSoldier, %targetLocation);

      %vector = vectorscale(%vector, %Fmultiplier);
      %x = Getword(%vector,0);
      %y = Getword(%vector,1);

      %vector = %x@" "@%y@" 0";
      %DiscSoldier.applyImpulse(%pos, %vector);

      //reached target, lets move on
      if(vectorDist(%pos, %targetLocation) < 75) {
         //%DiscSoldier.pressFire(30);
         DiscSoldierAttack(%DiscSoldier, %target);
         %DiscSoldier.attacking = 1;
         %DiscSoldier.Moving = 0;
         return;
      }

      //if(%DiscSoldier.getForwardVector() $= "0 0 0") { //old method
      %muzzlePos = %DiscSoldier.getEyePoint();
      %muzzleVec = %DiscSoldier.getEyeVector();
      %endPos    = VectorAdd(%muzzlePos, VectorScale(%muzzleVec, 2));
      %Mask = $TypeMasks::PlayerObjectType;

      //stuck?
      %hit = ContainerRayCast(%muzzlePos, %endPos, %Mask, %DiscSoldier);
      if(%hit) {
         %DiscSoldier.setVelocity(""@getRandom(-5,5)@" "@getRandom(-5,5)@" "@getRandom(1,5)@"");
         //now we arent :)
      }
   }
   %DiscSoldier.moveloop = schedule(500, %DiscSoldier, "DiscSoldierMove", %DiscSoldier, %targetLocation, %target);
}

function DiscSoldierAttack(%DiscSoldier, %target) {
   if(!isAlive(%target)) {
      %DiscSoldier.orders = 0;
      %DiscSoldier.attacking = 0;
      %DiscSoldier.Moving = 0;
      %DiscSoldier.setImageTrigger(0, false);
      return;
   }
   if(!isAlive(%DiscSoldier)) {
      return;
   }
   //look at the target
   %vec = GetDirectionToMove(%DiscSoldier, %target.getPosition());
   if(vectorDist(%DiscSoldier.getPosition(), %target.getPosition()) > 225) {    //too far away
      %DiscSoldier.orders = 0;
      %DiscSoldier.attacking = 0;
      %DiscSoldier.Moving = 0;
      %DiscSoldier.setImageTrigger(0, false);
      return;
   }
   else if(vectorDist(%DiscSoldier.getPosition(), %target.getPosition()) > 50) {
      %pos = %DiscSoldier.getworldboxcenter();
      %fmultiplier = %DiscSoldier.getDatablock().Speed;
      %vector = vectorscale(%vec, %Fmultiplier);
      %DiscSoldier.applyImpulse(%pos, %vector);
      %DiscSoldier.setImageTrigger(0, true);
      %DiscSoldier.setInventory(DiscAmmo, 99999, true);
      //serverPlay3d(ChaingunFireSound, %DiscSoldier.getPosition());
   }
   else {
      //manuvers
      %manu[1] = "5 0 0";   //Right
      %manu[2] = "-5 0 0";  //Left
      %manu[3] = "0 5 0";   //Forward
      %manu[4] = "0 -5 0";  //Backward
      %rnd = getRandom(1, 4); //pick a manuver
      //perfom it
      %pos = %DiscSoldier.getworldboxcenter();
      %fmultiplier = (%DiscSoldier.getDatablock().Speed) / 7.4;
      %vector = vectorscale(%manu[%rnd], %Fmultiplier);
      %DiscSoldier.applyImpulse(%pos, %vector);
      %DiscSoldier.setImageTrigger(0, true);
      %DiscSoldier.setInventory(DiscAmmo, 99999, true);
   }
   //
  // serverPlay3d(ChaingunFireSound, %DiscSoldier.getPosition());
   schedule(250, 0, "DiscSoldierAttack", %DiscSoldier, %target);
}

function DiscSoldierPatrolLoop(%DiscSoldier, %defendLocation) {
   if(!isAlive(%DiscSoldier)) {
	  return;
   }
   if(%DiscSoldier.Moving || %DiscSoldier.attacking) {
      %DiscSoldier.PatrolLoop = schedule(1000, 0, "DiscSoldierPatrolLoop", %DiscSoldier, %defendLocation);
   //   echo("moving | attacking");
      return;
   }
   else {
      if(vectorDist(%DiscSoldier.getPosition(), %defendLocation) > 15) {
         %DiscSoldier.Moving = 1;
         %DiscSoldier.Orders = 1;
        // echo("too far, returning");
         DiscSoldierMove(%DiscSoldier, %defendLocation, "");
      }
      else {
         %TargetSearchMask = $TypeMasks::PlayerObjectType;
         InitContainerRadiusSearch(%DiscSoldier.getPosition(), 200, %TargetSearchMask);
         while ((%potentialTarget = ContainerSearchNext()) != 0) {
            if (%potentialtarget && (%potentialTarget.team != %DiscSoldier.team)) {
               %DiscSoldier.Moving = 1;
               %DiscSoldier.Orders = 1;
               DiscSoldierMove(%DiscSoldier, %potentialTarget.getPosition(), %potentialTarget);
               //echo("target aquired");
            }
         }
      }
   }
   %DiscSoldier.PatrolLoop = schedule(1000, 0, "DiscSoldierPatrolLoop", %DiscSoldier, %defendLocation);
}
