//UNIT: SCOUT
//COST: 500, DEFCON 6
//DESCRIPTION: Scouts are fast infantry trained to seek out and report enemy bases

datablock PlayerData(UnitScoutArmor) : LightMaleBiodermArmor {
   maxDamage = 1.0;
   minImpactSpeed = 270;
   speedDamageScale = 0.00001;

   speed = 2600;
};

function SpawnScout(%client) {
   %Scout = new player(){
      Datablock = "UnitScoutArmor";
   };
   %name = "Scout";

   %Scout.name = %name;

   %Scout.target = createTarget(%Scout, %name, "", "Derm3", '', %Scout.team, PlayerSensor);
   setTargetName(%Scout.target, addtaggedstring(%name));
   setTargetSkin(%Scout.target, 'Horde');

   %Scout.BearingSet = 0;

   %client.OnDeployUnit(%Scout);
   
   schedule(500,0,"ScoutSeekLoop",%Scout);
}

function ScoutSeekLoop(%scout) {
   if(!isAlive(%scout)) {     //we still alive?
   //   echo("dead");
      return;
   }
   if(%scout.BearingSet) { //we are already moving to a site
   //   echo("bearing");
      schedule(3000, 0, "ScoutSeekLoop", %scout);
      return;
   }
   if(%Scout.ScanningObject == 1) { //we are looking at something
      schedule(3000, 0, "ScoutSeekLoop", %scout);
      return;
   }
   %goNext = RMPG(); //we can go anywhere!!!
   ScoutMove(%Scout, %goNext);
   schedule(3000, 0, "ScoutSeekLoop", %scout);
}

function ScoutMove(%Scout, %targetLocation) {
   if(!isAlive(%Scout)) {
	  return;
   }
   if(%Scout.ScanningObject == 1) { //we are looking at something
      return;
   }

   %pos = %Scout.getworldboxcenter();
   //echo(""@%pos@" - "@%targetLocation@" - "@vectorDist(%pos,%targetLocation)@"");
   %Scout.BearingSet = 1;
   %fmultiplier = %Scout.getDatablock().Speed;

   %vector = GetDirectionToMove(%Scout, %targetLocation);

   %vector = vectorscale(%vector, %Fmultiplier);
   %x = Getword(%vector,0);
   %y = Getword(%vector,1);

   %vector = %x@" "@%y@" 0";
   %Scout.applyImpulse(%pos, %vector);

   //reached target, lets move on
   if(vectorDist(%pos, %targetLocation) < 10) {
      %Scout.BearingSet = 0;
      return;
   }
   
   //Lets scan
   %TargetSearchMask = $TypeMasks::StaticShapeObjectType | $TypeMasks::VehicleObjectType;
   InitContainerRadiusSearch(%Scout.getPosition(),200,%TargetSearchMask);

   while ((%potentialTarget = ContainerSearchNext()) != 0){
      if (%potentialTarget.getPosition() != %Scout.getPosition()) {
         %Scout.ScanningObject = 1;
         if((%potentialTarget.team != %Scout.team) && %potentialTarget.MarkedBy[%Scout] == 0) {
            if(%potentialTarget.team == 0) {
               schedule(10000, 0, ResetScoutting, %scout);
               schedule(60000, 0, ClearMarker, %scout, %potentialTarget);
               //make the marker
               %potentialTarget.marker[%scout] = new BeaconObject() {
                  dataBlock = "DeployedBeacon";
                  position = %potentialTarget.getWorldBoxCenter();
                  team = %scout.team;
               };
               %potentialTarget.marker[%scout].setBeaconType(friend);
               %potentialTarget.marker[%scout].setTarget(%scout.team);
               MissionCleanup.add(%potentialTarget.marker[%scout]);
               %potentialTarget.MarkedBy[%Scout] = 1;
               %Scout.BearingSet = 0;
               return;
            }
            else {
               schedule(10000, 0, ResetScoutting, %scout);
               schedule(60000, 0, ClearMarker, %scout, %potentialTarget);
               %db = %potentialTarget.getDatablock().getName();
               if(%db $= "CommandPost" || %db $= "MissileSilo" || %db $= "OutpostVehicle") {
                  //make the marker
                  %potentialTarget.marker[%scout] = new BeaconObject() {
                     dataBlock = "DeployedBeacon";
                     position = %potentialTarget.getWorldBoxCenter();
                     team = %scout.team;
                  };
                  %potentialTarget.marker[%scout].setBeaconType(enemy);
                  %potentialTarget.marker[%scout].setTarget(%scout.team);
                  MissionCleanup.add(%potentialTarget.marker[%scout]);
                  %potentialTarget.MarkedBy[%Scout] = 1;
                  %Scout.BearingSet = 0;
                  return;
               }
               else {
                  %potentialTarget.MarkedBy[%Scout] = 1;
                  %Scout.ScanningObject = 0;
                  %Scout.BearingSet = 0;
               }
            }
         }
         else {
            %Scout.ScanningObject = 0;
         }
      }
      else {
         //it moved away
         %Scout.ScanningObject = 0;
      }
   }
   //

   //if(%Scout.getForwardVector() $= "0 0 0") { //old method
   %muzzlePos = %Scout.getEyePoint();
   %muzzleVec = %Scout.getEyeVector();
   %endPos    = VectorAdd(%muzzlePos, VectorScale(%muzzleVec, 2));
   %Mask = $TypeMasks::PlayerObjectType;

   //stuck?
   %hit = ContainerRayCast(%muzzlePos, %endPos, %Mask, %Scout);
   if(%hit) {
      %Scout.setVelocity(""@getRandom(-5,5)@" "@getRandom(-5,5)@" "@getRandom(1,5)@"");
      //now we arent :)
   }
   //}

   %Scout.moveloop = schedule(500, %Scout, "ScoutMove", %Scout, %targetLocation);
}

function ClearMarker(%scout, %target) {
   %target.MarkedBy[%Scout] = 0;
   if(isObject(%target.marker[%scout])) {
      %target.marker[%scout].delete();
   }
}

function ResetScoutting(%scout) {
   %Scout.ScanningObject = 0;
}

//ORDERS   (none, deselect)
function UnitScoutArmor::OnSelectUnit(%data, %scout, %obj) {
   %obj.SelectedUnit = "";
   BottomPrint(%obj.client, "Unit has pre-set Orders, De-Selected.", 1, 1);
}
