//UNIT: SEEKER
//COST: 500, DEFCON 4
//DESCRIPTION: Seekers can capture important objects

datablock PlayerData(SeekerArmor) : MediumMaleHumanArmor {
   maxDamage = 1.0;
   minImpactSpeed = 50;
   speedDamageScale = 0.015;

   speed = 2000;
};

function SpawnSeeker(%client) {
   %Seeker = new player(){
      Datablock = "SeekerArmor";
   };
   %name = "Seeker";

   %Seeker.name = %name;

   %Seeker.target = createTarget(%Seeker, %name, "", "Male3", '', %Seeker.team, PlayerSensor);
   setTargetName(%Seeker.target, addtaggedstring(%name));
   setTargetSkin(%Seeker.target, 'SWolf');

   %Seeker.BearingSet = 0;

   %client.OnDeployUnit(%Seeker);
}

function SeekerArmor::OnSelectUnit(%data, %seeker, %obj) {
   %seeker.nameToPush = "Seeker";
   %obj.order[0] = "Move [PTM]";
   %obj.order[1] = "Capture Object [PTSL]";
   %obj.order[2] = "Cancel Orders [PTO]";
}

function SeekerArmor::DisplayOrders(%data, %unit, %obj) {
   BottomPrint(%obj.client, "Unit: "@%unit.nameToPush@"("@%unit@") \n ORDER: "@%obj.order[%obj.TLMineMode]@"", 2, 2);
}

function SeekerArmor::ProcessOrders(%data, %seeker, %orders, %hitObj, %Position) {
   if(!isAlive(%seeker)) {
      return;
   }
   //
   if(%seeker.orders && %orders != 2) {  //we cannot do two things at once
      bottomPrint(%seeker.owner, "Cannot perform a second order, must cancel first", 1, 1);
      return;
   }
   else {
      switch(%orders) {
         case 0:
            %seeker.orders = 1;
            seekerMove(%seeker, %Position, "");
         case 1:
            %db = %hitObj.getDatablock().getName();
            echo(%db);
            if((%db !$= "ResourceSpire") || %hitObj.team == %seeker.team) {
               messageClient(%seeker.owner, 'msgInvalid', "\c5Seeker: I cannot capture that object");
               %seeker.orders = 0;
               return;
            }
            else {
               %seeker.orders = 1;
               seekerMove(%seeker, %Position, %hitObj);
            }
         case 2:
            %seeker.orders = 0;
         default:
            echo("Invalid orders");
            return;
      }
   }
}

//AI
function seekerMove(%seeker, %targetLocation, %capture) {
   if(!isAlive(%seeker)) {
	  return;
   }
   if(!%seeker.orders) {
      return;
   }
   %pos = %seeker.getworldboxcenter();
   //echo(""@%pos@" - "@%targetLocation@" - "@vectorDist(%pos,%targetLocation)@"");
   %fmultiplier = %seeker.getDatablock().Speed;

   %vector = GetDirectionToMove(%seeker, %targetLocation);

   %vector = vectorscale(%vector, %Fmultiplier);
   %x = Getword(%vector,0);
   %y = Getword(%vector,1);

   %vector = %x@" "@%y@" 0";
   %seeker.applyImpulse(%pos, %vector);

   //reached target, lets move on
   if(vectorDist(%pos, %targetLocation) < 10) {
      if(%capture !$= "" && %capture != 0) {
         if(!isObject(%capture)) {
            return;
         }
         else {
            SeekerCapture(%seeker, %capture);
         }
      }
      else {
         %seeker.orders = 0;
      }
      return;
   }

   //if(%seeker.getForwardVector() $= "0 0 0") { //old method
   %muzzlePos = %seeker.getEyePoint();
   %muzzleVec = %seeker.getEyeVector();
   %endPos    = VectorAdd(%muzzlePos, VectorScale(%muzzleVec, 2));
   %Mask = $TypeMasks::PlayerObjectType;

   //stuck?
   %hit = ContainerRayCast(%muzzlePos, %endPos, %Mask, %seeker);
   if(%hit) {
      %seeker.setVelocity(""@getRandom(-5,5)@" "@getRandom(-5,5)@" "@getRandom(1,5)@"");
      //now we arent :)
   }
   //}

   %seeker.moveloop = schedule(500, %seeker, "seekerMove", %seeker, %targetLocation, %capture);
}

function SeekerCapture(%seeker, %target) {
   %target.team = %seeker.team;
   setTargetSensorGroup(%target.target, %seeker.team);
   if(%target.getDatablock().getName() $= "ResourceSpire") {
      MessageAll('msgCapture', "\c5Defcon: "@%seeker.owner.namebase@" has captured a resource spire.");
      SpireLoop(%target, %seeker.owner);
   }
   freeclienttarget(%seeker);
   %seeker.delete();
}
